/* global Bdf,$$ */
/**
 * Objet global définissant l'espace de nom Ficheform
 * 
 * @namespace Ficheframe,Bdf
 */
var Ficheframe = {};

Ficheframe.backgroundColor = "#f4f4f4";
Ficheframe.startingLeft = 5;

Ficheframe.ARGS = {
    goto: ""
};

Ficheframe.checkFieldeditDim = function (fieldeditId) {
    Ficheframe.startingLeft = parseInt($$(fieldeditId).css("left"));
};

Ficheframe.testToolbar = function (iframe, toolbarId, fieldeditId, editTarget, tocTarget, callback) {
    var contentWindow = iframe.contentWindow;
    _showToolbar(false);
    var buttonId = "ficheframe_button_ficheedit";
    var pathName;
    try {
        pathName = contentWindow.location.pathname;
    } catch(error) {
        return;
    }
    if (!pathName.startsWith(_getFichothequeRoot())) {
        return;
    }
    var ficheTest = pathName.match(/\/fiches\/([a-z][a-z0-9]*)-([0-9]+)(-[_a-z0-9]+)?\.html$/);
    if (!ficheTest) {
        return;
    }
    var corpusName = ficheTest[1];
    var ficheId = ficheTest[2];
    var ficheblocks = null;
    Bdf.api.loadFiche({
        corpus: corpusName,
        id: ficheId,
        callback: function (data) {
            if (data.fiche) {
                let editable = data.fiche.editable;
                if (editable) {
                    _checkFields();
                } else {
                    $$(fieldeditId).html("");
                }
                let toc = _checkToc(contentWindow.document);
                let html = Bdf.render("ficheframe:toolbar", {
                    path: pathName,
                    corpusName: corpusName,
                    ficheId: ficheId,
                    editable: editable,
                    toc: toc,
                    editTarget: editTarget,
                    tocTarget: tocTarget
                });
                $$(toolbarId).html(html);
                if (html.length > 0) {
                    _showToolbar(true);
                    $(contentWindow).on('pagehide', function(e) {
                        _showToolbar(false);
                        _hideFieldEdit();
                    });
                    let editLink = $$.one(buttonId);
                    let originalHref = editLink.href;
                    if (toc.length > 0) {
                        $(contentWindow).on("scroll", function(event) {
                            let threshold = contentWindow.innerHeight * 2 / 3;
                            let scrollTop = $(contentWindow.document).scrollTop();
                            let currentTocItem = false;
                            for (let i = toc.length-1; i >= 0; i--) {
                                let tocItem = toc[i];
                                if (scrollTop > $(tocItem.element).offset().top - threshold) {
                                    currentTocItem = tocItem;
                                    if (i > 0) {
                                        let previousTocItem = toc[i - 1];
                                        let $previousEl = $(previousTocItem.element);
                                        let previousBottom = $previousEl.offset().top + $previousEl.height() + (threshold / 3);
                                        if (previousBottom > scrollTop) {
                                            currentTocItem = previousTocItem;
                                        }
                                    }
                                    break;
                                }
                            }
                            $(".ficheframe-toc-Active").removeClass("ficheframe-toc-Active");
                            if (currentTocItem) {
                                $$("linkto:" + currentTocItem.id).addClass("ficheframe-toc-Active");
                                editLink.href = originalHref + "&goto=" + currentTocItem.goto;
                            } else {
                                editLink.href = originalHref;
                            }
                        });
                    }
                }
                if (callback) {
                    callback();
                }
            }
        }
    });
    
    function _checkFields() {
        let timer;
        $(contentWindow).on("mousemove", function(event) {
            if (timer) {
                clearTimeout(timer);
                timer = false;
            }
            if (event.buttons) {
                _hideFieldEdit();
                return;
            }
            let currentSelection = contentWindow.getSelection();
            if ((currentSelection) && (currentSelection.type === "Range")) {
                _hideFieldEdit();
                return;
            }
            let target = event.target;
            if ((target) && (target.localName === "a")) {
                 _hideFieldEdit();
                return;
            }
            if ((ficheblocks) && (ficheblocks.isChanged(event.pageY))) {
                _hideFieldEdit();
            }
            if (event.pageX < 30) {
                __testPosition(event);
            } else {
                timer = setTimeout(function () {
                    __testPosition(event);
                }, 150);            
            }
        });
        $(contentWindow).on("scroll resize", function() {
            if (ficheblocks) {
                ficheblocks.clear();
                _hideFieldEdit();
                ficheblocks = null;
            }
        });
        
        function __testPosition(event) {
            let pageY = event.pageY;
            if (!ficheblocks) {
                ficheblocks = Ficheframe.Ficheblocks.build(contentWindow.document);
            }
            if (ficheblocks.isChanged(pageY)) {
                let newPosition = ficheblocks.updatePosition(pageY);
                if ((newPosition) && (newPosition.goto)) {
                    let top = newPosition.startY - $(contentWindow.document).scrollTop() + $(iframe).position().top;
                    if (top > 3) {
                        ficheblocks.activateCurrent(true);
                        $$(fieldeditId).html(Bdf.render("ficheframe:fieldedit", {
                            corpusName: corpusName,
                            ficheId: ficheId,
                            editTarget: editTarget,
                            goto: newPosition.goto
                        })).css("top", top).css("left", ___getLeft(newPosition));
                    }
                }
            }
            _checkFieldEdit();
            
            function ___getLeft(newPosition) {
                if (newPosition.startX > 50) {
                    return Ficheframe.startingLeft + newPosition.startX - 50;
                }
                return Ficheframe.startingLeft;
            }
        }
    }
    
    function _showToolbar(display) {
        Ficheframe.show(toolbarId, display);
    }
    
    function _hideFieldEdit() {
        Ficheframe.show(fieldeditId, false);
        if (ficheblocks) {
            ficheblocks.highlightCurrent(false);
        }
    }
    
    function _checkFieldEdit() {
        let display = ficheblocks.isCurrentActive;
        Ficheframe.show(fieldeditId, display);
        ficheblocks.highlightCurrent(display);
    }
    
    function _checkToc(document) {
        let array = new Array();
        $$(document, {bdfFormMilestone: true}).each(function (index, element) {
            let milestoneType = element.dataset.bdfFormMilestone;
            let goto;
            let $headings;
            if (milestoneType === "satellite") {
                goto = "satellite_" + element.dataset.bdfSatellite;
                $headings = $(element).children("h1");
            } else if (milestoneType === "comment") {
                goto = element.dataset.bdfComponentName;
                let $satellite = $(element).parents($$.toCssSelector({bdfFormMilestone: "satellite"}));
                if ($satellite.length > 0) {
                    goto = "satellite_" + $satellite[0].dataset.bdfSatellite + "/" + goto;
                }
                $headings = $(element).find("h2");
            }
            if ($headings) {
                $headings.each(function (index2, element2) {
                    let id = element2.id;
                    if (!id) {
                        id = Bdf.generateId();
                        element2.id = id;
                    }
                    let $element2 = $(element2);
                    let text = $element2.text();
                    if (!$element2.is(":hidden")) {
                        array.push({
                            id: id,
                            text: text,
                            element: element2,
                            goto: goto
                        });
                    }
                });
            }
        });
        return array;
    }
    
    function _getFichothequeRoot() {
        let currentPathName = window.location.pathname;
        let lastIndex = currentPathName.lastIndexOf("/");
        return currentPathName.substr(0, lastIndex + 1);
    }
};

Ficheframe.show = function (elementId, display) {
    if (display) {
        $$(elementId).removeClass("hidden");
    } else {
        $$(elementId).addClass("hidden");
    }
};